@@ -74,7 +74,7 @@ See `enable-circe-display-images'."
   :group 'circe)
 
 (defcustom circe-display-images-image-regex
-  "\\(https?:\/\/[^ ]*?\.\\\(?:png\\|jpg\\|jpeg\\|svg\\|gif\\)\\)"
+  "\\(https?://[^ ]*?\\.\\(?:png\\|jpg\\|jpeg\\|svg\\|gif\\)\\)"
   "Regex used to find images in channel messages. This regex needs to be
 greedy to match multiple images on the same line."
   :group 'circe-display-images
@@ -124,7 +124,7 @@ the image. See `circe-display-images-text-property-map' for more details."
   (interactive)
   (let*
       ((inhibit-read-only t)
-       (url (url-get-url-at-point))
+       (url (thing-at-point-url-at-point))
        (image-data(gethash url circe-display-images-text-property-map))
        (display-image-p (plist-get image-data :display-image-p))
        (image-property-of-url (plist-get image-data :image-property))
@@ -161,10 +161,7 @@ the image. See `circe-display-images-text-property-map' for more details."
                          (goto-char (point-min))
                          (search-forward "\n\n")
                          (buffer-substring (point) (point-max))))
-                 (img (create-image
-                       data 'imagemagick t
-                       :max-height circe-display-images-max-height
-                       :background circe-display-images-background)))
+                 (img (circe-create-image data)))
             (when img
               (insert-image img)
               ;; Store the image so that we can toggle it on and off later. We
@@ -179,6 +176,15 @@ the image. See `circe-display-images-text-property-map' for more details."
                 (image-animate img))))
         (kill-buffer buffer)))))
 
+(defconst circe-image-type (if (image-type-available-p 'imagemagick) 'imagemagick nil)
+  "Imagemagick image type if Emacs was compiled with support for ImageMagick, otherwise nil")
+
+(defun circe-create-image (data)
+  "Attempt to create image using Imagemagick type if available, otherwise guess type"
+  (create-image data circe-image-type t
+                :max-height circe-display-images-max-height
+                :background circe-display-images-background))
+
 (defun circe-display-images-urls-in-body ()
   "Return all urls that match the circe-display-images-image-regex"
   (let (urls)
