@@ -3,7 +3,7 @@
 ;; Copyright (C) 2015  Jorgen Schaefer <contact@jorgenschaefer.de>
 
 ;; Author: Jorgen Schaefer <contact@jorgenschaefer.de>
-;; URL: https://github.com/jorgenschaefer/circe
+;; URL: https://github.com/emacs-circe/circe
 
 ;; This program is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU General Public License
@@ -42,7 +42,11 @@
 ;;; Code:
 
 (require 'cl-lib)
-(require 'make-tls-process)
+(require 'gnutls)
+
+(if (fboundp 'gnutls-boot-parameters)
+    (declare-function 'make-tls-process 'make-tls-process)
+  (require 'make-tls-process))
 
 (defvar irc-debug-log nil
   "Emit protocol debug info if this is non-nil.")
@@ -70,23 +74,42 @@ conn.disconnected conn -- A previously established connection was lost
 
 NNN conn sender args... -- A numeric reply from IRC was received
 COMMAND conn sender args... -- An IRC command message was received"
-  (let ((proc (funcall (if (plist-get keywords :tls)
-                           #'make-tls-process
-                         #'make-network-process)
-                       :name (or (plist-get keywords :name)
-                                 (plist-get keywords :host))
-                       :host (or (plist-get keywords :host)
-                                 (error "Must specify a :host to connect to"))
-                       :service (or (plist-get keywords :service)
-                                    (error "Must specify a :service to connect to"))
+  (let* ((fun (if (and (plist-get keywords :tls)
+                       (not (fboundp 'gnutls-boot-parameters)))
+                  'make-tls-process
+                'make-network-process))
+         (host (or (plist-get keywords :host)
+                   (error "Must specify a :host to connect to")))
+         (service (or (plist-get keywords :service)
+                      (error "Must specify a :service to connect to")))
+         (tls-keylist (plist-get keywords :tls-keylist))
+         (tls-parameters (when (and (plist-get keywords :tls)
+                                    (fboundp 'gnutls-boot-parameters))
+                           (when (not (gnutls-available-p))
+                             (error "gnutls support missing"))
+                           (cons 'gnutls-x509pki
+                                 (gnutls-boot-parameters
+                                  :type 'gnutls-x509pki
+                                  :hostname host
+                                  :keylist tls-keylist))))
+         (proc (funcall fun
+                       :name (or (plist-get keywords :name) host)
+                       :host host
+                       :service service
                        :family (plist-get keywords :family)
                        :coding 'no-conversion
                        :nowait (featurep 'make-network-process '(:nowait t))
                        :noquery t
+                       :tls-parameters tls-parameters
                        :filter #'irc--filter
                        :sentinel #'irc--sentinel
                        :plist keywords
                        :keepalive t)))
+    (when (and (plist-get keywords :tls)
+               (fboundp 'nsm-verify-connection))
+      (setq proc (nsm-verify-connection proc host service))
+      (when (not proc)
+        (error "nsm verification failed")))
     ;; When we used `make-network-process' without :nowait, the
     ;; sentinel is not called with the open event, so we do this
     ;; manually.
@@ -98,6 +121,25 @@ COMMAND conn sender args... -- An IRC command message was received"
   "Return the value of CONN's PROPNAME property."
   (process-get conn propname))
 
+(defun irc-connection-uses-sasl-plain (conn)
+  "Return t if CONN uses sasl plain authentication"
+  (and (irc-connection-get conn :sasl-username)
+       (irc-connection-get conn :sasl-password)))
+
+(defun irc-connection-uses-sasl-external (conn)
+  "Return t if CONN uses sasl external authentication"
+  (irc-connection-get conn :sasl-external))
+
+(defun irc-connection-uses-sasl (conn)
+  "Return t if CONN uses sasl authentication"
+  (or (irc-connection-uses-sasl-plain conn)
+      (irc-connection-uses-sasl-external conn)))
+
+(defun irc-connection-uses-sasl-strict (conn)
+  "Return t if CONN uses sasl strict authentication"
+  (and (irc-connection-get conn :sasl-strict)
+       (irc-connection-uses-sasl conn)))
+
 (defun irc-connection-put (conn propname value)
   "Change CONN's PROPNAME property to VALUE."
   (process-put conn propname value))
@@ -373,7 +415,7 @@ they do not contain a space."
 (defun irc-send-AUTHENTICATE (conn arg)
   "Send an AUTHENTICATE message with ARG.
 
-See https://github.com/atheme/charybdis/blob/master/doc/sasl.txt
+See https://ircv3.net/specs/extensions/sasl-3.1
 for details."
   (irc-send-command conn "AUTHENTICATE" arg))
 
@@ -514,6 +556,9 @@ Connection options used:
 :cap-req -- CAP protocol capabilities to request, if available
 :sasl-username -- The SASL username to send, if sasl is available
 :sasl-password -- The SASL password to send, if sasl is available
+:sasl-external -- Use SASL external autentication, if sasl is available
+:sasl-strict -- If sasl is available and authentication failed leave
+   the server.
 
 Connection options set:
 
@@ -532,7 +577,17 @@ Connection options set:
   (irc-handler-add table "AUTHENTICATE"
                    #'irc-handle-registration--authenticate)
   (irc-handler-add table "900" ;; RPL_LOGGEDIN
-                   #'irc-handle-registration--logged-in))
+                   #'irc-handle-registration--logged-in)
+  (irc-handler-add table "903" ;; RPL_SASLSUCCESS
+                   #'irc-handle-registration--sasl-succeeded)
+  (irc-handler-add table "904" ;; ERR_SASLFAILED
+                   #'irc-handle-registration--sasl-failed)
+  (irc-handler-add table "905" ;; ERR_SASLTOOLONG
+                   #'irc-handle-registration--sasl-failed)
+  (irc-handler-add table "906" ;; ERR_SASLABORTED
+                   #'irc-handle-registration--sasl-aborted)
+  (irc-handler-add table "907" ;; ERR_SASLALREADY 
+                   #'irc-handle-registration--sasl-already))
 
 (defun irc-handle-registration--connected (conn _event)
   (irc-connection-put conn :connection-state 'connected)
@@ -570,10 +625,14 @@ Connection options set:
    ((equal subcommand "ACK")
     (let ((acked (split-string arg)))
       (irc-connection-put conn :cap-ack acked)
-      (if (and (member "sasl" acked)
-               (irc-connection-get conn :sasl-username)
-               (irc-connection-get conn :sasl-password))
-          (irc-send-AUTHENTICATE conn "PLAIN")
+      (if (member "sasl" acked)
+          (cond
+           ((irc-connection-uses-sasl-external conn)
+            (irc-send-AUTHENTICATE conn "EXTERNAL"))
+           ((irc-connection-uses-sasl-plain conn)
+            (irc-send-AUTHENTICATE conn "PLAIN"))
+           (t
+            (irc-send-CAP conn "END")))
         (irc-send-CAP conn "END"))))
    (t
     (message "Unknown CAP response from server: %s %s" subcommand arg))))
@@ -581,16 +640,43 @@ Connection options set:
 (defun irc-handle-registration--authenticate (conn _event _sender arg)
   (if (equal arg "+")
       (let ((username (irc-connection-get conn :sasl-username))
-            (password (irc-connection-get conn :sasl-password)))
-        (irc-send-AUTHENTICATE conn (base64-encode-string
-                                     (format "%s\x00%s\x00%s"
-                                             username username password)))
-        (irc-send-CAP conn "END"))
+            (password (irc-connection-get conn :sasl-password))
+            (external (irc-connection-get conn :sasl-external)))
+        (if external
+            (irc-send-AUTHENTICATE conn "+")
+            (irc-send-AUTHENTICATE conn (base64-encode-string
+                                         (format "%s\x00%s\x00%s"
+                                                 username username password)
+                                         t))))
     (message "Unknown AUTHENTICATE response from server: %s" arg)))
 
 (defun irc-handle-registration--logged-in (conn _event _sender _target
                                                 userhost account _message)
-  (irc-event-emit conn "sasl.login" userhost account))
+  (when (not (irc-connection-uses-sasl conn))
+    (irc-event-emit conn "sasl.login" userhost account)))
+
+(defun irc-handle-registration--sasl-succeeded (conn _event _sender target
+                                                     message)
+  (irc-send-CAP conn "END")
+  (irc-event-emit conn "sasl.login" target message))
+
+(defun irc-handle-registration--sasl-failed (conn _event _sender _target
+                                                  _message)
+  (cond
+   ((irc-connection-uses-sasl-strict conn)
+    (with-current-buffer (irc-connection-get conn :server-buffer)
+      (setq circe-server-inhibit-auto-reconnect-p t)
+      (irc-send-QUIT conn circe-default-quit-message)))
+   (t
+    (irc-send-CAP conn "END"))))
+
+(defun irc-handle-registration--sasl-aborted (conn _event _sender _target
+                                                   _message)
+  (irc-send-CAP conn "END"))
+
+(defun irc-handle-registration--sasl-already (conn _event _sender _target
+                                                   _message)
+  (irc-send-CAP conn "END"))
 
 (defun irc-connection-state (conn)
   "connecting connected registered disconnected"
