@@ -3,7 +3,7 @@
 ;; Copyright (C) 2005 - 2016  Jorgen Schaefer
 
 ;; Author: Jorgen Schaefer <forcer@forcix.cx>
-;; URL: https://github.com/jorgenschaefer/circe/wiki/Lui
+;; URL: https://github.com/emacs-circe/circe/wiki/Lui
 
 ;; This program is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 
 ;;; Code:
 
+(require 'cl-lib)
 (require 'button)
 (require 'flyspell)
 (require 'help-mode)
@@ -64,6 +65,7 @@
   :prefix "lui-"
   :group 'applications)
 
+(defvaralias 'lui-scroll-to-bottom-p 'lui-scroll-behavior)
 (defcustom lui-scroll-behavior t
   "Set the behavior lui should exhibit for scrolling.
 
@@ -108,7 +110,6 @@ behavior (post-output)."
                  (const :tag "Post Scroll" post-scroll)
                  (const :tag "Use default scrolling" nil))
   :group 'lui)
-(defvaralias 'lui-scroll-to-bottom-p 'lui-scroll-behavior)
 
 (defcustom lui-flyspell-p nil
   "Non-nil if Lui should spell-check your input.
@@ -449,8 +450,7 @@ Use `lui-insert' instead of accessing this marker directly.")
          (setq val (progn ,@body)))
        (when (consp buffer-undo-list)
          ;; Not t :-)
-         (lui-adjust-undo-list  ,old-marker-sym (- lui-input-marker
-                                                   ,old-marker-sym)))
+         (lui-adjust-undo-list (- lui-input-marker ,old-marker-sym)))
        val)))
 
 
@@ -470,7 +470,7 @@ It can be customized for an application by specifying a
         lui-output-marker (make-marker)
         lui-input-ring (make-ring lui-input-ring-size)
         lui-input-ring-index nil
-        flyspell-generic-check-word-p 'lui-flyspell-check-word-p)
+        flyspell-generic-check-word-predicate 'lui-flyspell-check-word-predicate)
   (set-marker lui-input-marker (point-max))
   (set-marker lui-output-marker (point-max))
   (add-hook 'window-scroll-functions 'lui-scroll-window nil t)
@@ -640,9 +640,11 @@ This uses `lui-buttons-list'."
                      (list (format lui-button-issue-tracker
                                    (match-string 2))))))))
 
+(defvar lui--buttonize-url-regex (regexp-opt thing-at-point-uri-schemes))
+
 (defun lui-buttonize-urls ()
   "Buttonize URLs in the current message."
-  (let ((regex (regexp-opt thing-at-point-uri-schemes)))
+  (let ((regex lui--buttonize-url-regex))
     (goto-char (point-min))
     (while (re-search-forward regex nil t)
       (let ((bounds (bounds-of-thing-at-point 'url)))
@@ -687,7 +689,9 @@ Otherwise, we move to the next button."
       (message "No such symbol %s" name)
       (ding))
      (t
-      (help-xref-interned sym)))))
+      (if (fboundp 'describe-symbol)
+          (describe-symbol sym)
+        (help-xref-interned sym))))))
 
 (defun lui-button-pep (number)
   "Browse the PEP NUMBER."
@@ -1010,38 +1014,26 @@ add to the inserted message."
 (defun lui--new-pos (pos shift)
   (* (if (< pos 0) -1 1) (+ (abs pos) shift)))
 
-(defun lui-adjust-undo-list (old-begin shift)
+(defun lui-adjust-undo-list (shift)
   ;; Translate buffer positions in buffer-undo-list by SHIFT.
   (unless (or (zerop shift) (atom buffer-undo-list))
     (let ((list buffer-undo-list) elt)
       (while list
         (setq elt (car list))
         (cond ((integerp elt)           ; POSITION
-               (if (lui--adjust-p elt old-begin)
-                   (setf (car list) (lui--new-pos elt shift))))
+               (cl-incf (car list) shift))
               ((or (atom elt)           ; nil, EXTENT
                    (markerp (car elt))) ; (MARKER . DISTANCE)
                nil)
               ((integerp (car elt))     ; (BEGIN . END)
-               (if (lui--adjust-p (car elt) old-begin)
-                   (setf (car elt) (lui--new-pos (car elt) shift)))
-               (if (lui--adjust-p (cdr elt) old-begin)
-                   (setf (cdr elt) (lui--new-pos (cdr elt) shift))))
+               (cl-incf (car elt) shift)
+               (cl-incf (cdr elt) shift))
               ((stringp (car elt))      ; (TEXT . POSITION)
-               (if (lui--adjust-p (cdr elt) old-begin)
-                   (setf (cdr elt) (lui--new-pos (cdr elt) shift))))
+               (cl-incf (cdr elt) (* (if (natnump (cdr elt)) 1 -1) shift)))
               ((null (car elt))         ; (nil PROPERTY VALUE BEG . END)
                (let ((cons (nthcdr 3 elt)))
-                 (if (lui--adjust-p (car cons) old-begin)
-                     (setf (car cons) (lui--new-pos (car cons) shift)))
-                 (if (lui--adjust-p (cdr cons) old-begin)
-                     (setf (cdr cons) (lui--new-pos (cdr cons) shift)))))
-              ((and (featurep 'xemacs)
-                    (extentp (car elt))) ; (EXTENT START END)
-               (if (lui--adjust-p (nth 1 elt) old-begin)
-                     (setf (nth 1 elt) (lui--new-pos (nth 1 elt) shift)))
-                 (if (lui--adjust-p (nth 2 elt) old-begin)
-                     (setf (nth 2 elt) (lui--new-pos (nth 2 elt) shift)))))
+                 (cl-incf (car cons) shift)
+                 (cl-incf (cdr cons) shift))))
         (setq list (cdr list))))))
 
 (defvar lui-prompt-map
@@ -1313,16 +1305,18 @@ If TEXT is specified, use that instead of formatting a new time stamp."
 
 (defun lui-time-stamp-enable-filtering ()
   "Enable filtering of timestamps from copied text."
-  (set (make-local-variable 'filter-buffer-substring-functions)
-       '(lui-filter-buffer-time-stamps)))
+  (set (make-local-variable 'filter-buffer-substring-function)
+       'lui-filter-buffer-time-stamps))
 
-(defun lui-filter-buffer-time-stamps (fun beg end delete)
+(defun lui-filter-buffer-time-stamps (beg end delete)
   "Filter text from copied strings.
 
 This is meant for the variable `filter-buffer-substring-functions',
 which see for an explanation of the argument FUN, BEG, END and
 DELETE."
-  (let ((string (funcall fun beg end delete))
+  (let ((string (if delete
+                    (delete-and-extract-region beg end)
+                  (buffer-substring beg end)))
         (inhibit-point-motion-hooks t)
         (inhibit-read-only t)
         ;; Emacs 24.4, 24.5
@@ -1340,15 +1334,6 @@ DELETE."
                                            'lui-time-stamp t))))
         (buffer-string)))))
 
-(defun lui-time-stamp-buffer-substring (buffer-string)
-  "Filter text from copied strings.
-
-This is meant for the variable `buffer-substring-filters',
-which see for an explanation of the argument BUFFER-STRING."
-  (lui-filter-buffer-time-stamps (lambda (_beg _end _delete)
-                                  buffer-string)
-                                nil nil nil))
-
 
 ;;;;;;;;;;;;;;;;;;
 ;;; Truncating ;;;
