@@ -9,31 +9,11 @@
 (describe "The `irc-connect' function"
   :var (process-status)
   (before-each
-    (spy-on 'make-tls-process :and-return-value 'the-test-tls-process)
+    (spy-on 'make-tls-process :and-return-value 'the-test-process)
     (spy-on 'make-network-process :and-return-value 'the-test-process)
     (spy-on 'process-status :and-call-fake (lambda (proc) process-status))
     (spy-on 'irc--sentinel :and-return-value nil))
 
-  (it "should call `make-network-process' if tls was not requested"
-    (irc-connect :host "irc.local"
-                 :service 6667)
-
-    (expect 'make-network-process
-            :to-have-been-called-with
-            :name "irc.local" :host "irc.local" :service 6667
-            :family nil
-            :coding 'no-conversion :nowait t :noquery t
-            :filter #'irc--filter :sentinel #'irc--sentinel
-            :plist '(:host "irc.local" :service 6667) :keepalive t))
-
-  (it "should call `make-tls-process' if tls was requested"
-    (irc-connect :host "irc.local"
-                 :service 6667
-                 :tls t)
-
-    (expect 'make-tls-process
-            :to-have-been-called))
-
   (it "should return a process when using non-tls connections"
     (expect (irc-connect :host "irc.local"
                          :service 6667)
@@ -43,7 +23,7 @@
     (expect (irc-connect :host "irc.local"
                          :service 6667
                          :tls t)
-            :to-be 'the-test-tls-process))
+            :to-be 'the-test-process))
 
   (it "should not use nowait if it is not supported"
     (spy-on 'featurep :and-return-value nil)
@@ -60,6 +40,7 @@
             :name "irc.local" :host "irc.local" :service 6667
             :family nil
             :coding 'no-conversion :nowait nil :noquery t
+            :tls-parameters nil
             :filter #'irc--filter :sentinel #'irc--sentinel
             :plist '(:host "irc.local" :service 6667) :keepalive t))
 
@@ -280,9 +261,6 @@
             '(nil "PRIVMSG" "#channel" "möp")))
 
   (it "should decode arguments individually"
-    ;; Undecided is broken in older Emacsen
-    (when (version< emacs-version "24.4")
-      (signal 'buttercup-pending t))
     ;; This is utf-16
     (expect (irc--parse
              (concat ":\xff\xfe\x6d\x00\xf6\x00\x70\x00 "
@@ -803,7 +781,7 @@
     (describe "on a CAP message"
       (it "should do the full negotiation"
         (irc-connection-put proc :cap-req '("multi-prefix"))
-        (irc-event-emit proc "conn.registered")
+        (irc-event-emit proc "irc.registered")
         (spy-calls-reset 'irc-send-raw)
         (irc-event-emit proc "CAP" "irc.server" "*" "LS" "multi-prefix")
         (irc-event-emit proc "CAP" "irc.server" "*" "ACK" "multi-prefix")
@@ -815,7 +793,7 @@
 
       (it "should not negotiation with no common capabilities"
         (irc-connection-put proc :cap-req '("sasl"))
-        (irc-event-emit proc "conn.registered")
+        (irc-event-emit proc "irc.registered")
         (spy-calls-reset 'irc-send-raw)
         (irc-event-emit proc "CAP" "irc.server" "*" "LS" "multi-prefix")
 
@@ -824,11 +802,11 @@
                 '("CAP END"))))
 
     (describe "on SASL authentication"
-      (it "should do the full negotiation"
+      (it "should start the negotiation"
         (irc-connection-put proc :cap-req '("sasl"))
         (irc-connection-put proc :sasl-username "my_nick")
         (irc-connection-put proc :sasl-password "top-secret")
-        (irc-event-emit proc "conn.registered")
+        (irc-event-emit proc "irc.registered")
         (spy-calls-reset 'irc-send-raw)
         (irc-event-emit proc "CAP" "irc.server" "*" "LS" "sasl")
         (irc-event-emit proc "CAP" "irc.server" "*" "ACK" "sasl")
@@ -838,8 +816,7 @@
                 :to-equal
                 '("CAP REQ sasl"
                   "AUTHENTICATE PLAIN"
-                  "AUTHENTICATE bXlfbmljawBteV9uaWNrAHRvcC1zZWNyZXQ="
-                  "CAP END"))))
+                  "AUTHENTICATE bXlfbmljawBteV9uaWNrAHRvcC1zZWNyZXQ="))))
 
     (describe "on SASL authentication"
       (it "should emit sasl.login for 900 numeric"
@@ -854,7 +831,70 @@
 
           (expect auth-args
                   :to-equal
-                  (list proc "sasl.login" "mynick!user@host" "account")))))))
+                  (list proc "sasl.login" "mynick!user@host" "account"))))
+
+      (it "should emit sasl.login for 903 numeric and stop the negotiation" 
+        (let (auth-args)
+          (irc-handler-add table "sasl.login"
+                           (lambda (&rest args)
+                             (setq auth-args args)))
+
+          (irc-event-emit proc "903" "irc.server" "mynick"
+                          "authentication successful")
+
+          (expect auth-args
+                  :to-equal
+                  (list proc "sasl.login" "mynick" "authentication successful"))
+
+          (expect (client-messages)
+                :to-equal
+                '("CAP END"))))
+
+      (it "should stop the negotiation for 904"
+        (irc-connection-put proc :sasl-username "my_nick")
+        (irc-connection-put proc :sasl-password "top-secret")
+        (irc-event-emit proc "904" "irc.server" "mynick"
+                        "authentication failed")
+
+        (expect (client-messages)
+                :to-equal
+                '("CAP END"))))
+
+    (it "should close the connection for 904 when sasl.strict is t"
+      (irc-connection-put proc :sasl-username "my_nick")
+      (irc-connection-put proc :sasl-password "top-secret")
+      (irc-connection-put proc :sasl-strict t)
+      (irc-connection-put proc :server-buffer (current-buffer))
+      (irc-event-emit proc "904" "irc.server" "mynick"
+                      "authentication failed")
+
+      (expect (client-messages)
+              :to-equal
+              '("QUIT :Using Circe, the loveliest of all IRC clients")))
+
+    (it "should stop the negotiation for 905"
+      (irc-event-emit proc "905" "irc.server" "mynick"
+                      ":SASL authentication failed")
+        
+      (expect (client-messages)
+              :to-equal
+              '("CAP END")))
+
+    (it "should stop the negotiation for 906"
+      (irc-event-emit proc "906" "irc.server" "mynick"
+                      ":SASL authentication aborted")
+      
+      (expect (client-messages)
+              :to-equal
+              '("CAP END")))
+
+    (it "should stop the negotiation for 907"
+      (irc-event-emit proc "907" "irc.server" "mynick"
+                      ":SASL already authenticated")
+      
+      (expect (client-messages)
+              :to-equal
+              '("CAP END")))))
 
 (describe "The `irc-connection-state' function"
   (let (proc)
@@ -1341,7 +1381,7 @@
               :to-equal
               '("#chan1" "#chan2" "#chan3")))
 
-    (it "should rember activity times for a rejoining user"
+    (it "should remember activity times for a rejoining user"
       (let ((channel (irc-channel-from-name proc "#channel"))
             user)
         (irc-channel-add-user channel "nick!user@host")
@@ -1354,7 +1394,7 @@
                  (irc-channel-user channel "nick"))
                 :to-equal 235)))
 
-    (it "should rember activity times for a user regaining their nick"
+    (it "should remember activity times for a user regaining their nick"
       (let ((channel (irc-channel-from-name proc "#channel"))
             user)
         (irc-channel-add-user channel "nick!user@host")
